function [matrixout0, matrixout1] = MAP2CanonicalExplicit (d0, d1) 

eps = 10^-8;

if size(d0,1)~=2
    if BuToolsVerbose
        fprintf ('MAP2Canonical: size is not 2!');
    end
	r=0;
    matrixout0 = 0;
    matrixout1 = 0;
	return;
end

if ~CheckMAP(d0, d1)
	r=0;
    matrixout0 = 0;
    matrixout1 = 0;
	return;
end


u=MarginalMomentsFromRAP(d0, d1, 3);
u1=u(1);
u2=u(2);
u3=u(3);
h1=u1;
h2=u2/(2*u1^2)-1;
h3=u3/(6*u1^3)-u2^2/(4*u1^4);

if abs(h2) < eps 
    if BuToolsVerbose
        fprintf ('MAP2Canonical: degenerate case: exponential marginal!');
    end
	r=0;
    matrixout0 = 0;
    matrixout1 = 0;
	return;
end

if h2>0
    lambda1=(h3+h2^2+h2-sqrt(((h3+h2^2+h2)^2)-4*h2*h3))/(2*h1*h3);
    lambda2=(h3+h2^2+h2+sqrt(((h3+h2^2+h2)^2)-4*h2*h3))/(2*h1*h3);
    p=(-h3-h2^2+h2+sqrt(((h3+h2^2+h2)^2)-4*h2*h3))/(h3+h2^2+h2+sqrt(((h3+h2^2+h2)^2)-4*h2*h3));
    alpha=lambda1/lambda2;
end

if h2<0
    lambda1=(h3+h2^2+h2+sqrt(((h3+h2^2+h2)^2)-4*h2*h3))/(2*h1*h3);
    lambda2=(h3+h2^2+h2-sqrt(((h3+h2^2+h2)^2)-4*h2*h3))/(2*h1*h3);
    p=(h3+h2^2-h2+sqrt(((h3+h2^2+h2)^2)-4*h2*h3))/(-h3-h2^2-h2+sqrt(((h3+h2^2+h2)^2)-4*h2*h3));
    alpha=lambda1/lambda2;
end

gamma=eig(inv(-d0)*d1);
gamma2=gamma(1);

if gamma2 >= 0
    %fprintf ('Positive corrrelation!');
    a = (1/(2*alpha))*(1+alpha*gamma2-p*(1-gamma2)-sqrt((1+alpha*gamma2-p*(1-gamma2))^2-4*alpha*gamma2));
    b = (1/2)*(1+alpha*gamma2-p*(1-gamma2)+sqrt((1+alpha*gamma2-p*(1-gamma2))^2-4*alpha*gamma2));
    g0=[-lambda1 (1-a)*lambda1; 0 -lambda2];
    g1=[a*lambda1 0; (1-b)*lambda2 b*lambda2];
    
end

if gamma2 < 0
    %fprintf ('Negative corrrelation!');
    a=-gamma2/(p*(1-gamma2)-alpha*gamma2);
    b=p*(1-gamma2)-alpha*gamma2;
    go=[-lambda1 (1-a)*lambda1; 0 -lambda2];
    g1=[0 a*lambda1; b*lambda2 (1-b)*lambda2];    
end

matrixout0=g0;
matrixout1=g1;