function [alpha, A] = APHFrom3Moments (m1, m2, m3)


% compute normalized moments
n1 = m1;
n2 = m2 / m1^2;
n3 = m3 / m1 / m2;

% detect number of phases needed
n = 2;
while n<100 && ((n+1) / n > n2 || lowerbound(n, n2) >= n3 || upperbound(n, n2) <= n3)
    n = n + 1;
end
if n2<(n+1)/n
    n2 = (n+1)/n;
end
if n3<lowerbound(n, n2)
    n3 = lowerbound(n, n2);
end
if n3>upperbound(n, n2)
    n3 = upperbound(n, n2);
end

if n2>2 || n3 < 2*n2 - 1
    b = 2*(4-n*(3*n2-4)) / (n2*(4+n-n*n3) + sqrt(n*n2)*sqrt(12*n2^2*(n+1)+16*n3*(n+1)+n2*(n*(n3-15)*(n3+1)-8*(n3+3))));
    a = (b*n2-2)*(n-1)*b / (b-1) / n;
    p = (b-1) / a;
    lambda = (p*a+1) / n1;
    mu = (n-1)*lambda / a;
    [alpha,A] = erlangexp (n, p, lambda, mu);
else
    c4 = n2*(3*n2-2*n3)*(n-1)^2;
    c3 = 2*n2*(n3-3)*(n-1)^2;
    c2 = 6*(n-1)*(n-n2);
    c1 = 4*n*(2-n);
    c0 = n*(n-2);
    fs = roots([c4 c3 c2 c1 c0]);
    found = 0;
    for i=1:length(fs)
        f = fs(i);
        a = 2*(f-1)*(n-1) / ((n-1)*(n2*f^2-2*f+2)-n);
        p = (f-1)*a;
        lambda = (a+p) / n1;
        mu = (n-1) / (n1 - p/lambda);
        if isreal(p) && isreal(lambda) && isreal(mu)&& p>=0 && p<=1 && lambda>0 && mu>0
            [alpha, A] = experlang (n, p, lambda, mu);
            found = 1;
        end
    end
    if found==0
        alpha = [];
        A = [];
    end
end

alpha = real(alpha);
A = real(A);
end

    function lb = lowerbound (n, n2)
        p = ((n+1)*(n2-2)) / (3*n2*(n-1)) * ((-2*sqrt(n+1)) / sqrt(-3*n*n2+4*n+4) -1);
        a = (n2-2) / (p*(1-n2) + sqrt(p^2+p*n*(n2-2)/(n-1)));
        l = ((3+a)*(n-1)+2*a) / ((n-1)*(1+a*p)) - (2*a*(n+1)) / (2*(n-1)+a*p*(n*a+2*n-2));
        if n2<(n+1)/n
            lb = Inf;
        elseif n2<(n+4)/(n+1)
            lb = l;
        else
            lb = (n+1)/n *n2;
        end
    end

    function ub = upperbound (n, n2)
        u = (2*(n-2)*(n*n2-n-1)*sqrt(1+(n*(n2-2))/(n-1)) + (n+2)*(3*n*n2-2*n-2)) / (n^2*n2);
        if n2<(n+1)/n
            ub = -Inf;
        elseif n2<=n/(n-1)
            ub = u;
        else
            ub = Inf;
        end
    end

    function [alpha,A] = erlangexp (n, p1, lambda1, mu1)
        alpha = zeros(1,n);
        alpha(1) = p1;
        alpha(n) = 1-p1;
        A = zeros(n,n);
        A(n,n) = -lambda1;
        for i=1:n-1
            A(i,i) = -mu1;
            A(i,i+1) = mu1;
        end
    end

    function [alpha,A] = experlang (n, p1, lambda1, mu1)
        alpha = zeros(1,n);
        alpha(1) = p1;
        alpha(2) = 1-p1;
        A = zeros(n,n);
        A(1,1) = -lambda1;
        A(1,2) = lambda1;
        for i=2:n
            A(i,i) = -mu1;
            if i<n
                A(i,i+1) = mu1;
            end
        end
    end
