function [alpha, A, r] = MomsToPH2 (moments)

m1 = moments(1);
m2 = moments(2);
m3 = moments(3);

alpha = [];
A = [];

if m2<1.5*m1^2
    fprintf ('The given second moment is not feasible!\n');
    r = -1;
    return;
elseif m2>=1.5*m1^2 && m2<=2*m1^2
    if m3>6*m1*(m2-m1^2)
        fprintf ('The given third moment is not feasible (too large)!\n');
        r = -2;
        return;
    elseif m3<(6*m2^2*(6*m1^2-2*m2+3*m1*sqrt(4*m1^2-2*m2))) / (2*m1+sqrt(4*m1^2-2*m2))^3
        fprintf ('The given third moment is not feasible (too low)!\n');
        r = -3;
        return;
    end
else
    delta = 0.05;
    if m3<3*m2^2 / (2* m1) + delta*m1*m2
        fprintf ('The given third moment is not feasible (too low)!\n');
        r = -4;
        return;
    end
end
r = 0;

%
% If we have an exponential distribution, we give an alternative form
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
c2 = m2 / m1^2 - 1;
if abs(c2-1) < 1e-6
    alpha = [1/2, 1-1/2];
    A = [-1/m1, 1/m1; 0, -2/m1];
    %alpha = [1/(2*c2), 1-1/(2*c2)];
    %A = [-1/(m1*c2), 1/(m1*c2); 0, -2/m1];
    return;
end


%
% Obtain the parameters of the APH(2)
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

b = 3*m1*m2-m3;
c = 3*m2^2-2*m1*m3;
e = -2*m1^2+m2;
a = b^2+6*c*e;
if a<0
    a = 0;
end

if c>0
    lambda1 = (b - sqrt(a)) / c;
    lambda2 = (b + sqrt(a)) / c;
    p = (-b-6*m1*e+sqrt(a)) / (b+sqrt(a));
elseif c<0
    lambda1 = (b + sqrt(a)) / c;
    lambda2 = (b - sqrt(a)) / c;
    p = (b+6*m1*e+sqrt(a)) / (-b+sqrt(a));
elseif c==0
    lambda1 = 0;
    lambda2 = 1 / m1;
    p = 0;
end

%
% Constuct transient generator and initial probability
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A = [-lambda1 lambda1; 0 -lambda2];
alpha = [p 1-p];
