function [alpha, T, r] = MomsToPH3 (moms)

    mean = moms(1);
    alpha = [];
    T = [];
    r = 0; % error code

    %convert the moments to factorial moments
    moms(1) = 1;
    fact = 1;
    for i=2:5
        fact = fact * i;
        moms(i) = moms(i) / fact / mean^i;
    end

    %solve linear system of equations for a0 a1 a2
    M = [moms(3) -moms(2) moms(1); moms(4) -moms(3) moms(2); moms(5) -moms(4) moms(3)];
    a = M\[1; moms(1); moms(2)];

    if a(3)^2-3*a(2)<0
        r = -1;
        return;
    end

    gu = (a(3) + 2*sqrt(a(3)^2-3*a(2))) / 3;
    g0 = (a(3) + sqrt(a(3)^2-3*a(2))) / 3;

    [tmp,ix] = sort(real(roots([1 fliplr(a')])),'ascend');

    lmb = -roots([1 fliplr(a')]);
    lambda = lmb(ix);

    d1 = a(2) - a(3) - a(1) * moms(2);
    d2 = a(1) - a(2) - a(3) * d1;
    d3 = -a(1) - a(2)*d1 - a(3)*d2;

    if d1>1e-10 || (abs(d1)<1e-10 && d2>0)
        r = -3;
        return;
    end

    if lambda(3)<0
        r = -5;
        return;
    end

    if imag(lambda(1)) < 1e-6
        gl = real(lambda(1));
    else
        gl = g0;
    end
    if gl>gu+1e-14
        r = -2;
        return;
    end
    if gl>gu
        gl = gu;
    end
    if abs(d1)<1e-10
        g2 = 0;
    else
        g2 = -d2 / d1;
    end

    if g2>gu+1e-9
        r = -4;
        return;
    end
    if g2>gu
        g2 = gu;
    end

    x1 = max(g2, gl);

    if real(lambda(1))==lambda(1) && g2<gl
        x13 = 0;
    else
          x13 = x1 - a(1) / (x1^2 - a(3)*x1 + a(2));
    end

    bels = (a(3)-x1)^2 - 4*(x1^2-a(3)*x1+a(2));
    if bels<0 && bels>-1e-12
        bels = 0;
    end

    x2 = (a(3) - x1 + sqrt(bels)) / 2;
    x3 = (a(3) - x1 - sqrt(bels)) / 2;
    p1 = d1 / (x13 - x1);
    p2 = (x1*d1 + d2) / (x13-x1) / x2;
    p3 = (x1*x2*d1 + x2*d2 + x1*d2 + d3) / (x13-x1) / x2 / x3;

    T = [-x1 0 x13; x2 -x2 0; 0 x3 -x3] / mean;
    alpha = [p1 p2 p3];

    if x13<-1e-9 || x13>x1
        r = -6;
        return;
    end

    if min(real(alpha))<-1e-10
        r = -7;
        return;
    end
    if max(abs(imag(alpha)))>1e-5
        r = -8;
        return;
    end

    if max(real(alpha))>1+1e-10
        r = -9;
        return;
    end
end
