%path(path, './utils')
%path(path, './ph')
%path(path, './map')
path(path, './fluid')

disp('matrix_analytic [Q, R, thres] -> [ pmatrix, f ] : Calculates the properties of the Markovian fluid queue with generator matrices Q, rate matrices R and threshold levels thres')

statenum=5; %cardinality of the statespace
thres=[0 0.5 1 1.5 2]; %the threshold levels, the first element corresponds to the bottom of the buffer
                       %the last element is the buffer-limit
thresnum=length(thres); %the number of thresholds
                       
Q=zeros(statenum,statenum,thresnum-1); %the generator matrix (it is a statenumstatenum size matrix for all thres-1 homogeneous parts of the system)
R=zeros(statenum,statenum,thresnum-1); %the rate matrix
Q(:,:,1)=[-2 1 1 0 0; 1 -3 2 0 0; 0 0 -2 1 1; 0 0 0 -1 1; 4 0 0 0 -4];
R(:,:,1)=[-2 0 0 0 0; 0 6 0 0 0; 0 0 -1 0 0; 0 0 0 1 0; 0 0 0 0 -1];
Q(:,:,2)=[-2 1 1 0 0; 1 -3 2 0 0; 0 0 -2 1 1; 0 0 0 -1 1; 4 0 0 0 -4];
R(:,:,2)=[-1 0 0 0 0; 0 5 0 0 0; 0 0 -3 0 0; 0 0 0 1 0; 0 0 0 0 -1];
Q(:,:,3)=[-2 1 1 0 0; 1 -3 2 0 0; 0 0 -2 1 1; 0 0 0 -1 1; 4 0 0 0 -4];
R(:,:,3)=[10 0 0 0 0; 0 10 0 0 0; 0 0 -3 0 0; 0 0 0 1 0; 0 0 0 0 -1];
Q(:,:,4)=[-2 1 1 0 0; 1 -3 2 0 0; 0 0 -2 1 1; 0 0 0 -1 1; 4 0 0 0 -4];
R(:,:,4)=[-10 0 0 0 0; 0 -10 0 0 0; 0 0 -3 0 0; 0 0 0 1 0; 0 0 0 0 -1];

%checking for random matrices with drifts close to 0
%{
Q=zeros(statenum,statenum,thresnum-1);
R=Q;
for l=1:thresnum-1
    Q(:,:,l)=rand(statenum);
    for k=1:statenum
        Q(k,k,l)=0;
    end
    Q(:,:,l)=Q(:,:,l)-diag(sum(Q(:,:,l),2));
    R(:,:,l)=diag(rand(statenum,1)-0.5);  
    %setting the drifts close to 0:
    stac=null(Q(:,:,l)')';
    drift=stac*diag(R(:,:,l));
    STACM=diag(stac)*statenum;
    %R(:,:,l)=R(:,:,l)-0.95*drift*STACM^(-1);
end

%}

%------------------matrix_analytic-----------------------------------------
[pmatrix, f, x]=matrix_analytic(Q, R, thres);

%plotting the results
figure(1)
plot(x,f);
xlabel('x')
ylabel('probability density')
axis([thres(1) thres(end) 0 1.2*max(f(:))]);


%----------------------additive_decomposition------------------------------
disp('additive_decomposition [Q, R, thres] -> [ pthres, f]: Calculates the properties of the Markovian fluid queue with generator matrices Q, rate matrices R and threshold levels thres')

[pmatrix, f, x]=additive_decomposition(Q,R,thres);

figure(2)
plot(x,f);
xlabel('x')
ylabel('probability density')
axis([thres(1) thres(end) 0 1.2*max(f(:))]);