path(path, './utils')
path(path, './ph')
path(path, './map')

global BuToolsVerbose
BuToolsVerbose = 1

disp('MarginalMomentsFromRAP [ matrix0, matrix1, k] -> [ moments ] : Calculates the first k marginal moments of the RAP with representation (matrix0, matrix1)')
disp('MarginalMomentsFromMAP [ matrix0, matrix1, k] -> [ moments ] : Calculates the first k marginal moments of the MAP with representation (matrix0, matrix1)')

d0=[-6.2 2 0; 2 -9 1; 1 0 -3]
d1=[2.2 -2 4;2 2 2;1 0 1]

m=MarginalMomentsFromRAP(d0, d1, 2)

disp('LagkJointMomentsFromRAP [ matrix0, matrix1, k ] -> [ moments ] : Calculates the matrix of the E(X_0^i,X_k^j) moments of the RAP with representation (matrix0, matrix1)')
d0=[-6.2 2 0; 2 -9 1; 1 0 -3]
d1=[2.2 -2 4;2 2 2;1 0 1]

disp('k=1')
Nm=LagkJointMomentsFromRAP(d0, d1, 1)
disp('k=2')
Nm=LagkJointMomentsFromRAP(d0, d1, 2)

disp('LagCorrelationsFromRAP [ matrix0, matrix1 ] -> [ lagcorrelations] : Calculates the lag correlations of the RAP with representation (matrix0, matrix1)')
d0=[-6.2 2 0; 2 -9 1; 1 0 -3]
d1=[2.2 -2 4;2 2 2;1 0 1]
lagcorrs=LagCorrelationsFromRAP(d0,d1)

disp('RAPToMAP [ matrix0, matrix1 ] -> [ outputmx0, outputmx1 ] : Similarity transforms the (matrix0, matrix1) representation to the Markovian representation (outputmx0, outputmx1)')
d0=[-5+1.2 2; 2 -9]
d1=[1-1.2 2; 3 4]
X={d0,d1};
Y = RationalToMarkovian (X, 2, @MAPTransFunc, @MAPEvalFunc, @MAPCheckFunc);
outputmx0=Y{1}
outputmx1=Y{2}

disp('StairCase [ matrix0, matrix1, closing vector ] -> [ size, simularity matrix ] : Computes a smaller representation of RAP using staircase algorithm ')
% Example 2 from the RAP paper
d0=[-5 1 0; 3 -3 0; 1 1 -5]
d1=[0 0 4; 0 0 0; 1 1 1]
cvec=[1;1;1]
ivec=[1/7 1/7 5/7]

u=Staircase(d0',d1',ivec')

c0=inv(u)*d0*u
c1=inv(u)*d1*u
icvec=ivec*u
ccvec=inv(u)*cvec

% Example 3 from the RAP paper
d0=[-5 1 0; 3 -3 0; 1 1 -4]
d1=[0 0 4; 0 0 0; 1 1 0]
cvec=[1;1;1]

u=Staircase(d0,d1,cvec)

c0=inv(u)*d0*u
c1=inv(u)*d1*u
ccvec=inv(u)*cvec

disp('MAP2CanonicalExplicit [matrix0, matrix1] -> [ matrix0, matrix1 ] : Calculates the canonical representation of the input MAP of size 2')
d0=[-14 1; 1 -25]
d1=[6 7; 3 21]
[matrix0, matrix1]=MAP2CanonicalExplicit(d0, d1)

disp('RAPFromMoments [ moments, nn ] -> [ outputmx0, outputmx1 ]: Calculates a RAP representation based on 2n-1 moments and the n*n matrix of the lag1 joint moments')
d0=[-6.2 2 0; 2 -9 1; 1 0 -3]
d1=[2.2 -2 4;2 2 2;1 0 1]
mom=MarginalMomentsFromRAP(d0, d1, 5);
nn=LagkJointMomentsFromRAP(d0, d1, 1);
[h0, h1] = RAPFromMoments(mom, nn)

disp('RAPMinimize [] ')
d0=[-5 1 0; 3 -3 0; 1 1 -5]
d1=[0 0 4; 0 0 0; 1 1 1]
[h0, h1]=RAPObsMinimize(d0,d1)
[h0, h1]=RAPContMinimize(d0,d1)
[h0, h1]=RAPMinimize(d0,d1)

disp('RAPMinimize [] ')
d0=[-5 1 0; 3 -3 0; 1 1 -4]
d1=[0 0 4; 0 0 0; 1 1 0]
[h0, h1]=RAPObsMinimize(d0,d1)
[h0, h1]=RAPContMinimize(d0,d1)
[h0, h1]=RAPMinimize(d0,d1)

