path(path, './utils')
path(path, './ph')

global BuToolsVerbose
BuToolsVerbose = 1

disp('MomentsFromME [ vector, matrix, k ] -> [ moments ] : Calculates the first k moments of a ME given with a vector-matrix pair')
alpha = [0.1 0.9]
A = [ -5+1.2 2; 2 -9 ]
[moms]=MomentsFromME(alpha, A, 2)

disp('PH3Canonical [ vector, matrix ] -> [ vector, matrix ] : Calculates the order 3 canonical representation from any order 3 vector-matrix representation, if exists')
alpha = [ 0.1 0.9 0]
A = [ -6.2 2 0; 2 -9 1; 1 0 -3 ]
[beta, B]=PH3Canonical(alpha, A)
[momsA]=MomentsFromME(alpha, A, 5)
[momsB]=MomentsFromME(beta, B, 5)

disp('MEFromMoments [ moments ] -> [ vector, matrix ] : Calculates the minimal ME representation based on 2n-1 moments')
alpha = [ 0.1 0.9 0]
A = [ -6.2 2 0; 2 -9 1; 1 0 -3 ]
momsA = MomentsFromME(alpha, A, 5)
[beta, B]=MEFromMoments(moms)
momsB = MomentsFromME(beta, B, 5)

disp('MomentsFromPH [ vector, matrix, k ] -> [ moments ] : Calculates the first k moments of a PH given with a vector-matrix pair')
alpha = [ 0.1 0.9 0]
A = [ -6.2 2 0; 2 -9 1; 1 0 -3]
moms = MomentsFromPH(alpha, A, 5)

disp('MonocyclicRepresentation [ vector, matrix ] -> [ vector, matrix ] : Calculates the representation of the input ME distribution with Markovian monocyclic generator')
A = [ -1 0.9 0.1 0 0 0; 1.5 -2 0.5 0 0 0; 0.5 0 -3 0.5 0 0; 0 0 0 -1 0.9 0.1; 0 0 0 1.5 -2 0.5; 0 0 0 0.5 0 -3]
alpha = [ -0.5 -0.2 1.2 -0.5 0.3 0.7]
[beta, B]=MonocyclicRepresentation(alpha, A)
momsA = MomentsFromME(alpha, A, 11)
momsB = MomentsFromME(beta, B, 11)

disp('APHFrom3Moments [ mom1, mom2, mom3] -> [ vector, matrix ] : Calculates the smallest APH with these 3 moments')
[init, QQ] = APHFrom3Moments(10, 125, 8400)
moms = MomentsFromPH(init, QQ, 5)

disp('METoPH [ vector, matrix ] -> [ outputvec, outputmx ] : Similarity transforms the (matrix0, matrix1) representation to a Markovian representation (outputvec, outputmx)')
alpha=[3.0 1-3.0]
d0=[-3 1; 3 -4]
X={alpha,d0};
Y = RationalToMarkovian (X, 2, @PHTransFunc, @PHEvalFunc, @PHCheckFunc);
outputvec=Y{1}
outputmx=Y{2}


disp('MEMoments [ vector, matrix ] -> [ moments ] : Calculates 2n-1 moments of the ME distribution from the vector-matrix pair of size n')
A = [-1 9/10 1/10; 15/10 -2 5/10; 25/10 0 -3]
alpha = [1 0 0]
moms=MEMoments(alpha, A)

disp('APHRepresentation [ vector, matrix ] -> [ vector, matrix ] : Calculates the APH (CF1) representation from any order n vector-matrix representation, if exists')
alpha = [1/3 2/3 -1/2]
A = [-4 1 1; 0 -2 1; 1 0 -8]
[beta, B]=APHRepresentation(alpha, A)


disp('MEContOrder [ vector, matrix ] -> [ order ] : Observability (initial vector) order of the vector-matrix pair ')
disp('MEObsOrder [ vector, matrix ] -> [ order ] : Controllability (closing vector) order of the vector-matrix pair ')
alpha=[1 1 1 1 1 1]/6
A=[-1 0 0 0 0 0; 0.5 -2 1 0 0 0; 1 0 -3 1 0 0; 1 0 1 -4 1 0; 4 0 0 0 -5 0; 5 0 0 0 0 -6]

contorder = MEContOrder(alpha, A)
obsorder = MEObsOrder(alpha, A)

disp('MEOrderFromMoments [ moments ] -> [ order ] : Calculates the order of the ME distribution based on its moments using the determinant of the Hankel matrix')
A = [-1 9/10 1/10; 15/10 -2 5/10; 25/10 0 -3];
alpha = [1 0 0];
moms=MEMoments(alpha, A)
order=MEOrderFromMoments(moms)

disp('CheckMEPositiveDensity [ vector, matrix ] -> [ flag ] : Checks it the vector-matrix pair results in a positive density')
alpha=[-2.5 0.9 2.6]
A=[-6.2 2 0; 2 -9 1; 1 0 -3]
CheckMEPositiveDensity(alpha, A)

%alpha=[-0.6 0.6 1]
%A=[-1.01 0.2 0; -0.2 -1.01 0; 0 0 -1]
