path(path, './utils')
path(path, './ph')
path(path, './map')
path(path, './specialprocesses')

global BuToolsVerbose
BuToolsVerbose = 1

disp('MARKED OR BATCH MARKOVIAN ARRIVAL PROCESS RELATED FUNCTIONS ==========================')

disp('MRAPContMinimize [ vector of matrix0, matrix1m ... matrixK ] -> [ vector of matrix0, matrix1m ... matrixK ] : Computes a minimal representation of an MRAP using the staircase method by eliminating the redundancy caused by the closing vector.')

disp('MRAPObsMinimize [ vector of matrix0, matrix1m ... matrixK ] -> [ vector of matrix0, matrix1m ... matrixK ] : Computes a minimal representation of an MRAP using the staircase method by eliminating the redundancy caused by the initial vector.')

disp('MRAPMinimize [ vector of matrix0, matrix1m ... matrixK ] -> [ vector of matrix0, matrix1m ... matrixK ] : Computes a minimal representation of an MRAP using the staircase method by eliminating the redundancy caused by the initial and the closing vector.')

XMX=[-5., 1, 0; 3, -3., 0; 1, 1, -5.];
XMX(:,:,2)=[0, 0, 4; 0, 0, 0; 1, 1, 1];
CMX=MRAPContMinimize(XMX)
OMX=MRAPObsMinimize(XMX)
MMX=MRAPMinimize(XMX)

XMX=[-5., 1, 0; 3, -3., 0; 1, 1, -4.];
XMX(:,:,2)=[0, 0, 4; 0, 0, 0; 1, 1, 0];
CMX=MRAPContMinimize(XMX)
OMX=MRAPObsMinimize(XMX)
MMX=MRAPMinimize(XMX)


XMX=[-6., 0, 0;0, -5., 2.;0, 0, -3.];
XMX(:,:,2)=[1, 0, 5; 0, 1, 1; 1, 0, 1];
XMX(:,:,3)=[0, 0, 0; 0, 1, 0; 0, 0, 1]
CMX=MRAPContMinimize(XMX)
OMX=MRAPObsMinimize(XMX)
MMX=MRAPMinimize(XMX)

disp('MRAPFromMoments [ marginal moments, vector of jointmomentsmatrix1, ... jointmomentsmatrixK ] -> [ vector of matrix0, matrix1m ... matrixK ] : Calculates an MRAP representation based on 2n-1 marginal moments and the nxn matrices of the lag 1 joint moments.')

disp('Lag1JointMomentsFromMRAP [ vector (1 x n), matrix0 (n x n), matrix1 (n x n^2) ] -> [ moments ] : returns: defective joint moments of X0 X1 from 0 to n-1')

disp('MomentsFromMRAP [ vector of matrix0, matrix1m ... matrixK ] -> [ moments ] : Calculates the first 2n-1 marginal moments of the MRAP of size n.')

XMX=[-5 0 1 1; 1 -8 1 0; 1 0 -4 1; 1 2 3 -9]
XMX(:,:,2)=[0 1 0 0; 0 1 2 0; 0 0 0 1; 1 0 0 0]
XMX(:,:,3)=[0 0 0 2; 2 0 1 0; 0 0 1 0; 0 1 0 1]

moms = MomentsFromMRAP(XMX)

JMOM = Lag1JointMomentsFromMRAP(XMX)

YMX = MRAPFromMoments(moms, JMOM)


disp('MRAPToMMAP [ vector of (matrix1, ..., matrixK) ] -> [ vector of (outputmx1, ..., outpurmxK) ] : Similarity transforms the (matrix0, ..., matrixK) representation to a Markovian representation ')
d0=[-5., 0.1+0.18, 0.9, 1 ;
    1, -8., 0.9,  0.1;
    0.9, 0.1, -4.,1;
    1, 2, 3, -9];
d1=[0.1-0.18, 0.7, 0.1,0.1; 
    0.1, 1, 1.8, 0.1; 
    0.1, 0.1, 0.1, 0.7;
    0.7, 0.1, 0.1, 0.1];
d2=[0.1, 0.1, 0.1, 1.7; 
    1.8, 0.1, 1, 0.1; 
    0.1, 0.1, 0.7, 0.1;
    0.1, 1, 0.1, 0.8];
X={d0,d1,d2};
Y = MRAPToMMAP(X);
outputd0=Y{1}
outputd1=Y{2}
outputd2=Y{3}



disp('TRANSIENT MARKOVIAN ARRIVAL PROCESS RELATED FUNCTIONS ==========================')


disp('MomentsFromTRAP [ vector (1 x n), matrix0 (n x n), matrix1 (n x n) ] -> [ moments ] :  returns: defective moments of X0 from 0 to 2n-1')
alpha=[0.6 0.4]
d0=[-4 0; 1 -6]
d1=[1 2; 3 0]
mom=MomentsFromTRAP(alpha, d0, d1)

disp('Lag1JointMomentsFromTRAP [ vector (1 x n), matrix0 (n x n), matrix1 (n x n^2) ] -> [ moments ] : returns: defective joint moments of X0 X1 from 0 to n-1')
alpha=[0.6 0.4]
d0=[-4 0; 1 -6]
d1=[1 2; 3 0]
nn = Lag1JointMomentsFromTRAP(alpha, d0, d1, 1)

disp('TRAPFromMoments [  vector (1 x 2n), matrix0 (n x n) ] -> [ vector (1 x n), matrix0 (n x n), matrix1 (n x n) ] :  input: X0 defective moments from 0 to 2n-1 + defective joint moments of X0 X1 from 0 to n-1')
mom=[0.7, 0.166667, 0.0805556, 0.0590278]
nn=[0.483333, 0.113889; 0.113889, 0.0266204]
[beta, g0, g1]=TRAPFromMoments(mom, nn)


disp('TRAPToTMAP [ vector of (vector, matrix1, matrix2) ] -> [ vector of (outputvec, outputmx1, outputmx2) ] : Similarity transforms the (matrix0, matrix1) representation to a Markovian representation ')
alpha=[0.3 1-0.3]
d0=[-5+1.2 2; 2 -9]
d1=[1-1.2 1; 2 3]
X={alpha,d0,d1};
Y = TRAPToTMAP(X);
outputvec=Y{1}
outputmx1=Y{2}
outputmx2=Y{3}



disp('MARKOVIAN BINARY TREE RELATED FUNCTIONS ==========================')


disp('MomentsFromRBT [ vector (1 x n), matrix0 (n x n), matrix1 (n x n^2) ] -> [ moments ] : returns: defective moments of X0 from 0 to 2n-1 ')
alpha=[0.6 0.4]
d0=[-4 0; 1 -6]
b=[1 1 0 1; 3 0 1 0]
mom=MomentsFromRBT(alpha, d0, b)

alpha=[0.1 0.5 0.4]
d0=[-14 0 1; 1 -6 0; 1 0 -3]
b=[1 1 0 1 2 0 0 0 1; 1 0 1 0 0 1 1 0 0; 0 0 0 0 0 0 0 0 1]
mom=MomentsFromRBT(alpha, d0, b)


disp('GammaijkMomentsFromRBT [ vector (1 x n), matrix0 (n x n), matrix1 (n x n^2) ] -> [ moments ] : calculates the defective joint moments of X0 X1 Y0 from 0 to n-1 in an n x n^2 matrix')
alpha=[0.6 0.4]
d0=[-4 0; 1 -6]
b=[1 1 0 1; 3 0 1 0]
nn=GammaijkMomentsFromRBT(alpha, d0, b)

disp('RBTFromMoments [  vector (1 x 2n), matrix0 (n x n^2) ] -> [ vector (1 x n), matrix0 (n x n), matrix1 (n x n^2) ] : input: X0 defective moments from 0 to 2n-1 + defective joint moments of X0 X1 Y0 from 0 to n-1. it returns a representation of the MBT')
mom=[ 0.613415, 0.141538, 0.0742387, 0.0650272, 0.0817178, 0.134025 ]
nn=[ 0.171487, 0.0430078, 0.0261052, 0.0373317, 0.00981029, 0.0061251, 0.020324, 0.00545656, 0.00346726; 0.0398141, 0.0102993, 0.00628155, 0.00935326, 0.00262795, 0.00167278, 0.00531802, 0.00156136, 0.00101777; 0.0210095, 0.00558839, 0.00342213, 0.00527305, 0.00155904, 0.00100597, 0.00310096, 0.000968833, 0.000641766]
[beta, g0, gg]=RBTFromMoments(mom, nn)

disp('RBTToMBT [ vector of (vector, matrix1, matrix2) ] -> [ vectro of (outputvec, outputmx1, outpurmx2) ] : Similarity transforms the (matrix0, matrix1) representation to a Markovian representation ')
alpha=[0.2 1-0.2]
d0=[-8+1.2 2; 2 -13]
b=[1-1.2 1 2 1; 1 2 3 3]
X={alpha,d0,b};
Y = RBTToMBT(X);
outputvec=Y{1}
outputmx1=Y{2}
outputmx2=Y{3}

