path(path, './utils')

disp('Enable the verbose messages with the BuToolsVerbose flag')
global BuToolsVerbose
BuToolsVerbose = 1

disp('CTMCSolve[ mx ] -> [ vector ] gives the steady state distribution of the CTMC with generator matrix x.')
q=[0.1 0.5 0.4; 0.9 0.1 0.0]
vector=CTMCSolve(q)

q=[0.1 0.5 0.4; 0.9 0.1 0.0; 0.9 0.1 0.0]
vector=CTMCSolve(q)

q=[-0.9 0.5 0.4; 0.9 -0.9 0.0; 0.3 0.3 -0.6]
vector=CTMCSolve(q)

disp('CRPSolve [mx] -> [ vector ] : gives the steady state distribution of the CRP (without generator matrix check)')
q=[-0.9 0.5 0.4; 0.9 -0.9 0.0; 0.3 0.3 -0.6]
vector=CRPSolve(q)

disp('DRPSolve [mx] -> [ vector ] : gives the steady state distribution of the DRP')
q=[-0.9 0.5 0.4; 0.9 -0.9 0.0; 0.3 0.3 -0.6]
vector=DRPSolve(q)

disp('DTMCSolve[ mx ] -> [ vector ] gives the steady state distribution of the DTMC with transition probability matrix mx.')
q=[0.1 0.5 0.4; 0.9 0.1 0.0; 0.3 -0.3 0.4]
vector=DTMCSolve(q)

q=[0.1 0.5 0.4; 0.9 0.1 0.0; 0.3 0.3 0.4]
vector=DTMCSolve(q)

disp('MomsToNormMoms [ moments ] -> [ moments ] makes normalized moments from moments')
moms = [1.2 5 8]
nm=MomsToNormMoms([1.2 5 8])

disp('NormMomsToMoms [ moments ] -> [ moments ] makes moments from normalized moments')
moms = [1.2 3.47222 1.33333]
m=NormMomsToMoms([1.2 3.47222 1.33333])

disp('CheckMAP [matrix0, matrix1] checks if the input matrixes define a continuous time MAP (The matrix0 and matrix1 are equal squrare matrixes, matrix0 is a transient generator matrix, matrix1 has only positive or zero elements, and the rowsum of matrix0+matrix1 is 0)')
d0 = [-1 0 1; 0 -2 0; 1 0 -3]
d1 = [-1 0 1 0; 0 -2 0 1; 1 0 -3 0; 1 2 2 1]
CheckMAP(d0, d1)

d0 = [-1 0 1; 0 -2 0; 1 0 -3]
d1 = [1 0 1 ; 0 2 0; 1 0 3]
CheckMAP(d0, d1)

d0 = [-3 0 1; 0 -2 0; 1 0 -5]
d1 = [1 0 1 ; 0 2 0; 1 0 3]
CheckMAP(d0, d1)

disp('CheckTransientGenerator [matrix] checks if the matrix is a valid transient generator matrix (the matrix is a square matrix, the diagonal of the matrix is negative, the matrix has positive or zero off-diagonal elements, the real part of the maximum absolute eigenvalue is less than zero) ')
q = [-0.9 0.5 0.4; 0.9 -0.9 0; 0.3 0.3 -0.6]
CheckTransientGenerator(q)

q = [-0.9 0.2 0.4; 0 -0.9 0.9; 0 0.6 -0.6]
CheckTransientGenerator(q)

q = [-0.9 0.2 0.4; 0.9 -0.9 0; 0.3 0.3 -0.6]
CheckTransientGenerator(q)

disp('CheckGenerator [ matrix ] checks if the matrix is a valid generator matrix (the matrix is a square matrix, the matrix has positive or zero off-diagonal elements, the diagonal of the matrix is negative, the rowsum of the matrix is 0) ')
q = [-0.1 0.5 0.4; 0.9 -0.1 0; 0.3 0.3 -0.4]
CheckGenerator(q)

q = [-0.9 0.5 0.4; 0.9 -0.9 0; 0.3 0.3 -0.6]
CheckGenerator(q)

disp('CheckProbMatrix [ matrix ] checks if the matrix is a valid probability matrix (the matrix is a square matrix, the matrix has positive or zero off-diagonal elements, the rowsum of the matrix is 1)')
q = [0.1 0.5 0.4; 0.9 0.1 0; 0.3 -0.1 0.4]
CheckProbMatrix(q)

q = [0.1 0.5 0.4; 0.9 0.1 0; 0.3 0.1 0.4]
CheckProbMatrix(q)

q = [0.1 0.5 0.4; 0.9 0.1 0; 0.3 0.3 0.4]
CheckProbMatrix(q)

disp('CheckTransientProbMatrix [ matrix ] checks if the matrix is a valid transient probability matrix (the matrix is a square matrix, the matrix has positive or zero off-diagonal elements, the rowsum of the matrix is less equal than 1, the maximum absolute eigenvalue is less than 1)')
q = [0.1 0.5 0.4; 0.9 0.1 0; 0.3 0.3 0.4]
CheckTransientProbMatrix(q)

q = [0.1 0.5 0.4; 0.9 0.1 0; 0.3 0.1 0.4]
CheckTransientProbMatrix(q)

disp('CheckProbVector [ vector ] checks if the vector is a valid probability vector (the vector has only non-negative elements, the sum of the vector elements is 1)')
alpha = [1.1 -0.1]
CheckProbVector(alpha)

alpha = [1.1 0.1]
CheckProbVector(alpha)

alpha = [1 0]
CheckProbVector(alpha)

disp('CheckSubProbVector [ vector ] checks if the vector is a valid sub-probability vector (the vector has only non-negative elements, the sum of the vector elements is less equal than 1)')
alpha = [0.9 -0.1]
CheckSubProbVector(alpha)

alpha = [0.9 0.1]
CheckSubProbVector(alpha)

alpha = [0.8 0.1]
CheckSubProbVector(alpha)

disp('KroneckerProduct [ matrix0, matrix1 ] -> [ matrix ] gives the Kronecker product of the two matrices')
p1 = [ 0.1 0.5 0.4; 0.9 0.1 0.0; 0.3 0.3 0.4 ]
p2 = [ 1 0; 0.5 0.5 ]
C=KroneckerProduct(p1, p2)

disp('KroneckerSum [ matrix0, matrix1 ] -> [matrix ] gives the Kronecker sum of the two matrices')
q1 = [ -0.9 0.5 0.4; 0.9 -0.9 0; 0.3 0.3 -0.6 ]
q2 = [ -1 1; 1 -2]
C=KroneckerSum(q1, q2)

disp('ReducedmomsToMoms [ moments ] -> [ moments ] computes moments from reduced moments')
rmoms = [1 1 1 1 1]
m=ReducedmomsToMoms(rmoms)

disp('MomsToReducedmoms [ moments ] -> [ moments ] computes reduced moments from moments')
moms = [1 2 6 24 120]
rm=MomsToReducedmoms(moms)

disp('CheckPH [ vector, matrix ] checks the phase type conditions on a vector and a matrix (the vector is a probability vector, the matrix is a transient generator and they have the same size)')
alpha = [0.2]
A= [-1 1; 1 -2]
CheckPH(alpha, A)

alpha = [0.2 0.8]
A= [-1 1; 1 -2]
CheckPH(alpha, A)


disp('CheckME [ vector, matrix ] checks some matrix exponential conditions of a vector-matrix pair (the matrix is a square matrix, the vector and the matrix have the same size, the dominant eigenvalue is negative and real)')
alpha=[-0.2 0.2]
A=[1 -1; 1 -2]
CheckME(alpha, A)

alpha = [-0.2 0.2 0.3]
A=[-2 0 0; 0 -1 1; 0 -1 -1]
CheckME(alpha, A)

alpha = [-0.2 0.2]
A = [-1 1; 1 -2]
CheckME(alpha, A)
